const Auth = require("../models/auth");
const User = require("../models/user");
const Queue = require("../models/queue");
const TableTypes = require("../models/tabletypes");

const tableTypesModel = new TableTypes();
const userModel = new User();
const queueModel = new Queue();

const showReserve = async(req, res, next) => {
    const authResult = await Auth.getSessionData(req.session.token); 

    const queueResult = await queueModel.findOneBy("user_id",authResult.userId);
    if(queueResult) {
        res.redirect("/queue/track");
        return;
    }

    const tableTypesResult = await tableTypesModel.findAll();

    res.render("queue/index",{ role: authResult.role,tableTypesResult: tableTypesResult });
};

const showTrack = async(req, res, next) => {
    const authResult = await Auth.getSessionData(req.session.token); 

    const queueResult = await queueModel.findOneBy("user_id",authResult.userId);
    if(!queueResult) {
        res.redirect("/queue");
        return;
    }

    res.render("queue/track",{ role: authResult.role, queueid: queueResult.id });
};

const fetchQueue = async(req, res, next) => {
    const { id } = req.params;
    let queueResult = await queueModel.findOneJoinTableTypesByID(id);
    if(!queueResult) {
        return res.status(404).send("Not Found.");
    }
    const queueLeftResult = await queueModel.countQueue(id,queueResult.table_type);

    const date = new Date(queueResult.created_at);
    const buddhaYear = date.getFullYear() + 543;
    const formattedDate = `${date.getDate().toString().padStart(2, '0')}/${
        (date.getMonth() + 1).toString().padStart(2, '0')
    }/${buddhaYear} ${date.getHours().toString().padStart(2, '0')}:${
        date.getMinutes().toString().padStart(2, '0')
    }:${date.getSeconds().toString().padStart(2, '0')}`;

    queueResult.created_at = formattedDate;

    res.json({ queueData: queueResult,queueLeft: !queueLeftResult ? 0 : queueLeftResult.count });
};

const queueSubmit = async(req, res, next) => {
    const { size, tabletype } = req.body;
    const authResult = await Auth.getSessionData(req.session.token); 
    const queueResult = await queueModel.findOneBy("user_id",authResult.userId);
    if(queueResult) {
        res.redirect("/queue/track");
        return;
    }
    const insertResult = await queueModel.create({ table_type: tabletype, size: size, user_id: authResult.userId });

    res.redirect("/queue/track");
};

module.exports = {
    showReserve,
    showTrack,
    fetchQueue,
    queueSubmit,
};