const Auth = require("../models/auth");
const Role = require("../models/role");
const Staff = require("../models/staff");
const User = require("../models/user");

const staffModel = new Staff();
const userModel = new User();
const roleModel = new Role();

const showIndex = async(req, res, next) => {
    const { query } = req.query;
    const authResult = await Auth.getSessionData(req.session.token); 

    if(!query) {
        staffResult = await staffModel.findAllLeftJoinUser() || [ ];
    }
    else staffResult = await staffModel.searchLeftJoin(query) || [ ];

    const formattedResults = staffResult.map(item => {
        const date = new Date(item.created_at);
        const buddhaYear = date.getFullYear() + 543;
        const formattedDate = `${date.getDate().toString().padStart(2, '0')}/${
            (date.getMonth() + 1).toString().padStart(2, '0')
        }/${buddhaYear} ${date.getHours().toString().padStart(2, '0')}:${
            date.getMinutes().toString().padStart(2, '0')
        }:${date.getSeconds().toString().padStart(2, '0')}`;

        return {
            ...item,
            created_at: formattedDate,
        };
    });

    res.render("restaurant/staff/index",{ role: authResult.role, staffData: formattedResults });
};

const showCreate = async(req, res, next) => {
    const authResult = await Auth.getSessionData(req.session.token); 
    const rolesResult = await roleModel.findAll();
    const usersResult = await userModel.findAll() || [ ];

    res.render("restaurant/staff/create",{ role: authResult.role, rolesData: rolesResult,usersData: usersResult });
};

const showManage = async(req, res, next) => {
    const { id } = req.params;
    const authResult = await Auth.getSessionData(req.session.token); 
    const staffResult = await staffModel.findOneLeftJoinUser(id);
    const usersResult = await userModel.findAll() || [ ];
    if(staffResult == null) return res.status(404).send("Not Found.");
    const rolesResult = await roleModel.findAll();

    res.render("restaurant/staff/manage",{ role: authResult.role,staffData: staffResult,rolesData: rolesResult,usersData: usersResult,isSelf: staffResult.user_id == authResult.userId ? true : false });
};

const createStaff = async(req, res, next) => {
    const { name,role,user_id } = req.body;

    await staffModel.create({ name, role, user_id });

    res.redirect("/staff");
};

const updateStaff = async(req, res, next) => {
    const { id,name,role,user_id } = req.body;
    const staffResult = await staffModel.findOneBy('id',id);
    if(staffResult == null) return res.status(404).send("Not Found.");

    await staffModel.updateBy('id',id,{ name,role: !role || role == undefined ? staffResult.role : role,user_id: user_id == 0 ? null : user_id });

    res.redirect("/staff/manage/" + id);
};

const deleteStaff = async(req, res, next) => {
    const { id } = req.body;

    const authResult = await Auth.getSessionData(req.session.token); 

    await staffModel.deleteById(id);
    if(id == authResult.userId) {
        return res.status(400).send("You cannot delete yourself.");
    }


    res.redirect("/staff");
};

module.exports = {
    showIndex,
    showCreate,
    showManage,
    createStaff,
    updateStaff,
    deleteStaff,
};