const db = require("../database");

class SQLModel {
    constructor(tableName) {
        this.tableName = tableName;
    };
    async create(data) {
        const [result] = await db.mysql.query(`INSERT INTO \`${this.tableName}\` SET ? `, data);
        return result.insertId;
    }
    async findAll(orderBy = '') {
        let query = `SELECT * FROM \`${this.tableName}\` WHERE 1`;

        if (orderBy) {
            query += ` ORDER BY ${orderBy}`;
        }

        const [result] = await db.mysql.query(query);
        if (!result || result.length == 0) return null;
        return result;
    }
    async findAllJoin(joinTable, joinCondition, orderBy = '') {
        let query = `
            SELECT * 
            FROM \`${this.tableName}\`
            JOIN \`${joinTable}\`
            ON ${joinCondition}
        `;
    
        if (orderBy) {
            query += ` ORDER BY ${orderBy}`;
        }
    
        const [result] = await db.mysql.query(query);
        if (!result || result.length == 0) return null;
        return result;
    }
    async findAllLeftJoin(joinTable, joinCondition, orderBy = '', filterIds = [], filterTable = '', whereCondition = '') {
        let query = `
            SELECT * 
            FROM \`${this.tableName}\`
            LEFT JOIN \`${joinTable}\`
            ON ${joinCondition}
        `;
        
        if (filterIds.length > 0) {
            const placeholders = filterIds.map(() => '?').join(', ');
            query += ` WHERE \`${filterTable ? filterTable : joinTable}\`.id IN (${placeholders})`;
        }
    
        if (whereCondition) {
            query += filterIds.length > 0 ? ` AND ${whereCondition}` : ` WHERE ${whereCondition}`;
        }
    
        if (orderBy) {
            query += ` ORDER BY ${orderBy}`;
        }
    
        const [result] = await db.mysql.query(query, filterIds);
        if (!result || result.length == 0) return null;
        return result;
    }
    
    async findOneLeftJoin(joinTable, joinCondition, orderBy = '', filterIds = [],filterTable = '') {
        let query = `
            SELECT * 
            FROM \`${this.tableName}\`
            LEFT JOIN \`${joinTable}\`
            ON ${joinCondition}
        `;
        
        if (filterIds.length > 0) {
            const placeholders = filterIds.map(() => '?').join(', ');
            query += ` WHERE \`${filterTable ? filterTable : joinTable}\`.id IN (${placeholders})`;
        }
    
        if (orderBy) {
            query += ` ORDER BY ${orderBy}`;
        }
    
        const [result] = await db.mysql.query(query, filterIds);
        if (!result || result.length == 0) return null;
        return result[0];
    }

    async findAllBy(field,value) {
        const [result] = await db.mysql.query(`SELECT * FROM \`${this.tableName}\` WHERE \`${field}\` IN (?)`,[ value ]);
        if(!result || result.length == 0) return null;
        return result; 
    }
    async findOneBy(field,value) {
        const [result] = await db.mysql.query(`SELECT * FROM \`${this.tableName}\` WHERE \`${field}\` = ?`,[ value ]);
        if(!result || result.length == 0) return null;
        return result[0];
    }
    async findOneByJoin(field, value, joinTable, joinCondition) {
        const query = `
            SELECT * 
            FROM \`${this.tableName}\`
            JOIN \`${joinTable}\`
            ON ${joinCondition}
            WHERE ${field} = ?
        `;
        const [result] = await db.mysql.query(query, [value]);
        if (!result || result.length == 0) return null;
        return result[0];
    }
    async searchBy(field, value, orderBy = '') {
        let query = `SELECT * FROM \`${this.tableName}\` WHERE \`${field}\` LIKE ?`;
        
        if (orderBy) {
            query += ` ORDER BY ${orderBy}`;
        }
        
        const [result] = await db.mysql.query(query, [`%${value}%`]);
        if (!result || result.length == 0) return null;
        return result;
    }
    
    async updateBy(idField,idValue, updates) {
        const fields = [];
        const values = [];
        
        for (const [field, value] of Object.entries(updates)) {
            fields.push(`${field} = ?`);
            values.push(value);
        }
    
        const fieldString = fields.join(', ');
        
        values.push(idValue);
        
        const query = `UPDATE \`${this.tableName}\` SET ${fieldString} WHERE \`${idField}\` = ?`;
        
        const [result] = await db.mysql.query(query, values);
        
        return result;
    }
    async deleteById(Id) {
        const [result] = await db.mysql.query(`DELETE FROM \`${this.tableName}\` WHERE id = ?`, [ Id ]);
        if(!result || result.length == 0) return null;
        return result;   
    }    
};

module.exports = SQLModel;